@@ -35,15 +35,15 @@
 //! [`TracerProvider`]: crate::trace::TracerProvider
 
 use crate::global;
-use crate::runtime::Runtime;
+use crate::sdk::trace::runtime::{TraceRuntime, TrySend};
 use crate::sdk::trace::Span;
 use crate::{
     sdk::export::trace::{ExportResult, SpanData, SpanExporter},
     trace::{TraceError, TraceResult},
     Context,
 };
-use futures::{channel::mpsc, channel::oneshot, executor, future::Either, pin_mut, StreamExt};
-use std::{env, fmt, str::FromStr, sync::Mutex, thread, time::Duration};
+use futures::{channel::oneshot, executor, future::Either, pin_mut, StreamExt};
+use std::{env, fmt, str::FromStr, thread, time::Duration};
 
 /// Delay interval between two consecutive exports.
 const OTEL_BSP_SCHEDULE_DELAY: &str = "OTEL_BSP_SCHEDULE_DELAY";
@@ -171,6 +171,24 @@ impl SpanProcessor for SimpleSpanProcessor {
 /// A [`SpanProcessor`] that asynchronously buffers finished spans and reports
 /// them at a preconfigured interval.
 ///
+/// Batch span processors need to run a background task to collect and send
+/// spans. Different runtimes need different ways to handle the background task.
+///
+/// Note: Configuring an opentelemetry `Runtime` that's not compatible with the
+/// underlying runtime can cause deadlocks (see tokio section).
+///
+/// ### Use with Tokio
+///
+/// Tokio currently offers two different schedulers. One is
+/// `current_thread_scheduler`, the other is `multiple_thread_scheduler`. Both
+/// of them default to use batch span processors to install span exporters.
+///
+/// Tokio's `current_thread_scheduler` can cause the program to hang forever if
+/// blocking work is scheduled with other tasks in the same runtime. To avoid
+/// this, be sure to enable the `rt-tokio-current-thread` feature in this crate
+/// if you are using that runtime (e.g. users of actix-web), and blocking tasks
+/// will then be scheduled on a different thread.
+///
 /// # Examples
 ///
 /// This processor can be configured with an [`executor`] of your choice to
@@ -191,9 +209,7 @@ impl SpanProcessor for SimpleSpanProcessor {
 ///     // Configure your preferred exporter
 ///     let exporter = apitrace::NoopSpanExporter::new();
 ///
-///     // Then build a batch processor. You can use whichever executor you have available, for
-///     // example if you are using `async-std` instead of `tokio` you can replace the spawn and
-///     // interval functions with `async_std::task::spawn` and `async_std::stream::interval`.
+///     // Create a batch span processor using an exporter and a runtime
 ///     let batch = sdktrace::BatchSpanProcessor::builder(exporter, runtime::Tokio)
 ///         .with_max_queue_size(4096)
 ///         .build();
@@ -211,11 +227,11 @@ impl SpanProcessor for SimpleSpanProcessor {
 /// [`executor`]: https://docs.rs/futures/0.3/futures/executor/index.html
 /// [`tokio`]: https://tokio.rs
 /// [`async-std`]: https://async.rs
-pub struct BatchSpanProcessor {
-    message_sender: Mutex<mpsc::Sender<BatchMessage>>,
+pub struct BatchSpanProcessor<R: TraceRuntime> {
+    message_sender: R::Sender,
 }
 
-impl fmt::Debug for BatchSpanProcessor {
+impl<R: TraceRuntime> fmt::Debug for BatchSpanProcessor<R> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("BatchSpanProcessor")
             .field("message_sender", &self.message_sender)
@@ -223,21 +239,13 @@ impl fmt::Debug for BatchSpanProcessor {
     }
 }
 
-impl SpanProcessor for BatchSpanProcessor {
+impl<R: TraceRuntime> SpanProcessor for BatchSpanProcessor<R> {
     fn on_start(&self, _span: &Span, _cx: &Context) {
         // Ignored
     }
 
     fn on_end(&self, span: SpanData) {
-        let result = self
-            .message_sender
-            .lock()
-            .map_err(|_| TraceError::Other("batch span processor mutex poisoned".into()))
-            .and_then(|mut sender| {
-                sender
-                    .try_send(BatchMessage::ExportSpan(span))
-                    .map_err(|err| TraceError::Other(err.into()))
-            });
+        let result = self.message_sender.try_send(BatchMessage::ExportSpan(span));
 
         if let Err(err) = result {
             global::handle_error(err);
@@ -245,9 +253,9 @@ impl SpanProcessor for BatchSpanProcessor {
     }
 
     fn force_flush(&self) -> TraceResult<()> {
-        let mut sender = self.message_sender.lock().map_err(|_| TraceError::from("When force flushing the BatchSpanProcessor, the message sender's lock has been poisoned"))?;
         let (res_sender, res_receiver) = oneshot::channel();
-        sender.try_send(BatchMessage::Flush(Some(res_sender)))?;
+        self.message_sender
+            .try_send(BatchMessage::Flush(Some(res_sender)))?;
 
         futures::executor::block_on(res_receiver)
             .map_err(|err| TraceError::Other(err.into()))
@@ -255,9 +263,9 @@ impl SpanProcessor for BatchSpanProcessor {
     }
 
     fn shutdown(&mut self) -> TraceResult<()> {
-        let mut sender = self.message_sender.lock().map_err(|_| TraceError::from("When shutting down the BatchSpanProcessor, the message sender's lock has been poisoned"))?;
         let (res_sender, res_receiver) = oneshot::channel();
-        sender.try_send(BatchMessage::Shutdown(res_sender))?;
+        self.message_sender
+            .try_send(BatchMessage::Shutdown(res_sender))?;
 
         futures::executor::block_on(res_receiver)
             .map_err(|err| TraceError::Other(err.into()))
@@ -265,23 +273,26 @@ impl SpanProcessor for BatchSpanProcessor {
     }
 }
 
+/// Messages sent between application thread and batch span processor's work thread.
 #[derive(Debug)]
-enum BatchMessage {
+pub enum BatchMessage {
+    /// Export spans, usually called when span ends
     ExportSpan(SpanData),
+    /// Flush the current buffer to the backend, it can be triggered by
+    /// pre configured interval or a call to `force_push` function.
     Flush(Option<oneshot::Sender<ExportResult>>),
+    /// Shut down the worker thread, push all spans in buffer to the backend.
     Shutdown(oneshot::Sender<ExportResult>),
 }
 
-impl BatchSpanProcessor {
-    pub(crate) fn new<R>(
+impl<R: TraceRuntime> BatchSpanProcessor<R> {
+    pub(crate) fn new(
         mut exporter: Box<dyn SpanExporter>,
         config: BatchConfig,
         runtime: R,
-    ) -> Self
-    where
-        R: Runtime,
-    {
-        let (message_sender, message_receiver) = mpsc::channel(config.max_queue_size);
+    ) -> Self {
+        let (message_sender, message_receiver) =
+            runtime.batch_message_channel(config.max_queue_size);
         let ticker = runtime
             .interval(config.scheduled_delay)
             .map(|_| BatchMessage::Flush(None));
@@ -362,16 +373,13 @@ impl BatchSpanProcessor {
         }));
 
         // Return batch processor with link to worker
-        BatchSpanProcessor {
-            message_sender: Mutex::new(message_sender),
-        }
+        BatchSpanProcessor { message_sender }
     }
 
     /// Create a new batch processor builder
-    pub fn builder<E, R>(exporter: E, runtime: R) -> BatchSpanProcessorBuilder<E, R>
+    pub fn builder<E>(exporter: E, runtime: R) -> BatchSpanProcessorBuilder<E, R>
     where
         E: SpanExporter,
-        R: Runtime,
     {
         BatchSpanProcessorBuilder {
             exporter,
@@ -388,7 +396,7 @@ async fn export_with_timeout<R, E>(
     batch: Vec<SpanData>,
 ) -> ExportResult
 where
-    R: Runtime,
+    R: TraceRuntime,
     E: SpanExporter + ?Sized,
 {
     if batch.is_empty() {
@@ -487,7 +495,7 @@ pub struct BatchSpanProcessorBuilder<E, R> {
 impl<E, R> BatchSpanProcessorBuilder<E, R>
 where
     E: SpanExporter + 'static,
-    R: Runtime,
+    R: TraceRuntime,
 {
     /// Set max queue size for batches
     pub fn with_max_queue_size(self, size: usize) -> Self {
@@ -528,7 +536,7 @@ where
     }
 
     /// Build a batch processor
-    pub fn build(self) -> BatchSpanProcessor {
+    pub fn build(self) -> BatchSpanProcessor<R> {
         BatchSpanProcessor::new(Box::new(self.exporter), self.config, self.runtime)
     }
 }
