@@ -7,25 +7,91 @@
 //! It contains two identifiers - a `TraceId` and a `SpanId` - along with a set of common
 //! `TraceFlags` and system-specific `TraceState` values.
 //!
-//! The spec can be viewed here: https://github.com/open-telemetry/opentelemetry-specification/blob/master/specification/api-tracing.md#SpanContext
+//! The spec can be viewed here: <https://github.com/open-telemetry/opentelemetry-specification/blob/v1.3.0/specification/trace/api.md#spancontext>
 //!
 //! [w3c TraceContext specification]: https://www.w3.org/TR/trace-context/
 #[cfg(feature = "serialize")]
 use serde::{Deserialize, Serialize};
 use std::collections::VecDeque;
+use std::fmt;
+use std::ops::{BitAnd, BitOr, Not};
 use std::str::FromStr;
+use thiserror::Error;
 
-/// A SpanContext with TRACE_FLAG_NOT_SAMPLED means the span is not sampled.
-pub const TRACE_FLAG_NOT_SAMPLED: u8 = 0x00;
-/// TRACE_FLAG_SAMPLED is a bitmask with the sampled bit set. A SpanContext
-/// with the sampling bit set means the span is sampled.
-pub const TRACE_FLAG_SAMPLED: u8 = 0x01;
-/// TRACE_FLAGS_DEFERRED is a bitmask with the deferred bit set. A SpanContext
-/// with the deferred bit set means the sampling decision has been
-/// defered to the receiver.
-pub const TRACE_FLAG_DEFERRED: u8 = 0x02;
-/// TRACE_FLAGS_DEBUG is a bitmask with the debug bit set.
-pub const TRACE_FLAG_DEBUG: u8 = 0x04;
+/// Flags that can be set on a [`SpanContext`].
+///
+/// The current version of the specification only supports a single flag called `sampled`.
+///
+/// See the W3C TraceContext specification's [trace-flags] section for more details.
+///
+/// [trace-flags]: https://www.w3.org/TR/trace-context/#trace-flags
+#[cfg_attr(feature = "serialize", derive(Deserialize, Serialize))]
+#[derive(Clone, Debug, Default, PartialEq, Eq, Copy, Hash)]
+pub struct TraceFlags(u8);
+
+impl TraceFlags {
+    /// Trace flags with the `sampled` flag set to `1`.
+    ///
+    /// Spans that are not sampled will be ignored by most tracing tools.
+    /// See the `sampled` section of the [W3C TraceContext specification] for details.
+    ///
+    /// [W3C TraceContext specification]: https://www.w3.org/TR/trace-context/#sampled-flag
+    pub const SAMPLED: TraceFlags = TraceFlags(0x01);
+
+    /// Construct new trace flags
+    pub const fn new(flags: u8) -> Self {
+        TraceFlags(flags)
+    }
+
+    /// Returns `true` if the `sampled` flag is set
+    pub fn is_sampled(&self) -> bool {
+        (*self & TraceFlags::SAMPLED) == TraceFlags::SAMPLED
+    }
+
+    /// Returns copy of the current flags with the `sampled` flag set.
+    pub fn with_sampled(&self, sampled: bool) -> Self {
+        if sampled {
+            *self | TraceFlags::SAMPLED
+        } else {
+            *self & !TraceFlags::SAMPLED
+        }
+    }
+
+    /// Returns the flags as a `u8`
+    pub fn to_u8(self) -> u8 {
+        self.0
+    }
+}
+
+impl BitAnd for TraceFlags {
+    type Output = Self;
+
+    fn bitand(self, rhs: Self) -> Self::Output {
+        Self(self.0 & rhs.0)
+    }
+}
+
+impl BitOr for TraceFlags {
+    type Output = Self;
+
+    fn bitor(self, rhs: Self) -> Self::Output {
+        Self(self.0 | rhs.0)
+    }
+}
+
+impl Not for TraceFlags {
+    type Output = Self;
+
+    fn not(self) -> Self::Output {
+        Self(!self.0)
+    }
+}
+
+impl fmt::LowerHex for TraceFlags {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::LowerHex::fmt(&self.0, f)
+    }
+}
 
 /// TraceId is an 16-byte value which uniquely identifies a given trace
 /// The actual `u128` value is wrapped in a tuple struct in order to leverage the newtype pattern
@@ -173,16 +239,15 @@ impl TraceState {
     /// # Examples
     ///
     /// ```
-    /// use opentelemetry::trace::TraceState;
+    /// use opentelemetry::trace::{TraceState, TraceStateError};
     ///
     /// let kvs = vec![("foo", "bar"), ("apple", "banana")];
-    /// let trace_state: Result<TraceState, ()> = TraceState::from_key_value(kvs);
+    /// let trace_state: Result<TraceState, TraceStateError> = TraceState::from_key_value(kvs);
     ///
     /// assert!(trace_state.is_ok());
     /// assert_eq!(trace_state.unwrap().header(), String::from("foo=bar,apple=banana"))
     /// ```
-    #[allow(clippy::all)]
-    pub fn from_key_value<T, K, V>(trace_state: T) -> Result<Self, ()>
+    pub fn from_key_value<T, K, V>(trace_state: T) -> Result<Self, TraceStateError>
     where
         T: IntoIterator<Item = (K, V)>,
         K: ToString,
@@ -192,14 +257,16 @@ impl TraceState {
             .into_iter()
             .map(|(key, value)| {
                 let (key, value) = (key.to_string(), value.to_string());
-                if !TraceState::valid_key(key.as_str()) || !TraceState::valid_value(value.as_str())
-                {
-                    return Err(());
+                if !TraceState::valid_key(key.as_str()) {
+                    return Err(TraceStateError::InvalidKey(key));
+                }
+                if !TraceState::valid_value(value.as_str()) {
+                    return Err(TraceStateError::InvalidValue(value));
                 }
 
                 Ok((key, value))
             })
-            .collect::<Result<VecDeque<_>, ()>>()?;
+            .collect::<Result<VecDeque<_>, TraceStateError>>()?;
 
         if ordered_data.is_empty() {
             Ok(TraceState(None))
@@ -227,13 +294,20 @@ impl TraceState {
     /// updated key/value is returned.
     ///
     /// ['spec']: https://www.w3.org/TR/trace-context/#list
-    #[allow(clippy::all)]
-    pub fn insert(&self, key: String, value: String) -> Result<TraceState, ()> {
-        if !TraceState::valid_key(key.as_str()) || !TraceState::valid_value(value.as_str()) {
-            return Err(());
+    pub fn insert<K, V>(&self, key: K, value: V) -> Result<TraceState, TraceStateError>
+    where
+        K: Into<String>,
+        V: Into<String>,
+    {
+        let (key, value) = (key.into(), value.into());
+        if !TraceState::valid_key(key.as_str()) {
+            return Err(TraceStateError::InvalidKey(key));
+        }
+        if !TraceState::valid_value(value.as_str()) {
+            return Err(TraceStateError::InvalidValue(value));
         }
 
-        let mut trace_state = self.delete(key.clone())?;
+        let mut trace_state = self.delete_from_deque(key.clone());
         let kvs = trace_state.0.get_or_insert(VecDeque::with_capacity(1));
 
         kvs.push_front((key, value));
@@ -242,26 +316,29 @@ impl TraceState {
     }
 
     /// Removes the given key-value pair from the `TraceState`. If the key is invalid per the
-    /// [W3 Spec]['spec'] or the key does not exist an `Err` is returned. Else, a new `TraceState`
+    /// [W3 Spec]['spec'] an `Err` is returned. Else, a new `TraceState`
     /// with the removed entry is returned.
     ///
+    /// If the key is not in `TraceState`. The original `TraceState` will be cloned and returned.
     /// ['spec']: https://www.w3.org/TR/trace-context/#list
-    #[allow(clippy::all)]
-    pub fn delete(&self, key: String) -> Result<TraceState, ()> {
+    pub fn delete<K: Into<String>>(&self, key: K) -> Result<TraceState, TraceStateError> {
+        let key = key.into();
         if !TraceState::valid_key(key.as_str()) {
-            return Err(());
+            return Err(TraceStateError::InvalidKey(key));
         }
 
+        Ok(self.delete_from_deque(key))
+    }
+
+    /// Delete key from trace state's deque. The key MUST be valid
+    fn delete_from_deque(&self, key: String) -> TraceState {
         let mut owned = self.clone();
-        let kvs = owned.0.as_mut().ok_or(())?;
-
-        if let Some(index) = kvs.iter().position(|x| *x.0 == *key) {
-            kvs.remove(index);
-        } else {
-            return Err(());
+        if let Some(kvs) = owned.0.as_mut() {
+            if let Some(index) = kvs.iter().position(|x| *x.0 == *key) {
+                kvs.remove(index);
+            }
         }
-
-        Ok(owned)
+        owned
     }
 
     /// Creates a new `TraceState` header string, delimiting each key and value with a `=` and each
@@ -285,7 +362,7 @@ impl TraceState {
 }
 
 impl FromStr for TraceState {
-    type Err = ();
+    type Err = TraceStateError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let list_members: Vec<&str> = s.split_terminator(',').collect();
@@ -293,7 +370,7 @@ impl FromStr for TraceState {
 
         for list_member in list_members {
             match list_member.find('=') {
-                None => return Err(()),
+                None => return Err(TraceStateError::InvalidList(list_member.to_string())),
                 Some(separator_index) => {
                     let (key, value) = list_member.split_at(separator_index);
                     key_value_pairs
@@ -306,13 +383,33 @@ impl FromStr for TraceState {
     }
 }
 
+/// Error returned by `TraceState` operations.
+#[derive(Error, Debug)]
+#[non_exhaustive]
+pub enum TraceStateError {
+    /// The key is invalid. See https://www.w3.org/TR/trace-context/#key for requirement for keys.
+    #[error("{0} is not a valid key in TraceState, see https://www.w3.org/TR/trace-context/#key for more details")]
+    InvalidKey(String),
+
+    /// The value is invalid. See https://www.w3.org/TR/trace-context/#value for requirement for values.
+    #[error("{0} is not a valid value in TraceState, see https://www.w3.org/TR/trace-context/#value for more details")]
+    InvalidValue(String),
+
+    /// The value is invalid. See https://www.w3.org/TR/trace-context/#list for requirement for list members.
+    #[error("{0} is not a valid list member in TraceState, see https://www.w3.org/TR/trace-context/#list for more details")]
+    InvalidList(String),
+}
+
 /// Immutable portion of a `Span` which can be serialized and propagated.
+///
+/// Spans that do not have the `sampled` flag set in their [`TraceFlags`] will
+/// be ignored by most tracing tools.
 #[cfg_attr(feature = "serialize", derive(Deserialize, Serialize))]
 #[derive(Clone, Debug, PartialEq)]
 pub struct SpanContext {
     trace_id: TraceId,
     span_id: SpanId,
-    trace_flags: u8,
+    trace_flags: TraceFlags,
     is_remote: bool,
     trace_state: TraceState,
 }
@@ -323,7 +420,7 @@ impl SpanContext {
         SpanContext::new(
             TraceId::invalid(),
             SpanId::invalid(),
-            0,
+            TraceFlags::default(),
             false,
             TraceState::default(),
         )
@@ -333,7 +430,7 @@ impl SpanContext {
     pub fn new(
         trace_id: TraceId,
         span_id: SpanId,
-        trace_flags: u8,
+        trace_flags: TraceFlags,
         is_remote: bool,
         trace_state: TraceState,
     ) -> Self {
@@ -358,7 +455,7 @@ impl SpanContext {
 
     /// Returns details about the trace. Unlike `TraceState` values, these are
     /// present in all traces. Currently, the only option is a boolean sampled flag.
-    pub fn trace_flags(&self) -> u8 {
+    pub fn trace_flags(&self) -> TraceFlags {
         self.trace_flags
     }
 
@@ -373,19 +470,11 @@ impl SpanContext {
         self.is_remote
     }
 
-    /// Returns if the deferred bit is set in the trace flags
-    pub fn is_deferred(&self) -> bool {
-        (self.trace_flags & TRACE_FLAG_DEFERRED) == TRACE_FLAG_DEFERRED
-    }
-
-    /// Returns if the debug bit is set in the trace flags
-    pub fn is_debug(&self) -> bool {
-        (self.trace_flags & TRACE_FLAG_DEBUG) == TRACE_FLAG_DEBUG
-    }
-
-    /// Returns true if the `SpanContext` is sampled.
+    /// Returns `true` if the `sampled` trace flag is set.
+    ///
+    /// Spans that are not sampled will be ignored by most tracing tools.
     pub fn is_sampled(&self) -> bool {
-        (self.trace_flags & TRACE_FLAG_SAMPLED) == TRACE_FLAG_SAMPLED
+        self.trace_flags.is_sampled()
     }
 
     /// Returns the context's `TraceState`.
@@ -454,7 +543,7 @@ mod tests {
 
             let new_key = format!("{}-{}", test_case.0.get(test_case.2).unwrap(), "test");
 
-            let updated_trace_state = test_case.0.insert(test_case.2.into(), new_key.clone());
+            let updated_trace_state = test_case.0.insert(test_case.2, new_key.clone());
             assert!(updated_trace_state.is_ok());
             let updated_trace_state = updated_trace_state.unwrap();
 
@@ -473,4 +562,12 @@ mod tests {
             assert!(deleted_trace_state.get(test_case.2).is_none());
         }
     }
+
+    #[test]
+    fn test_trace_state_insert() {
+        let trace_state = TraceState::from_key_value(vec![("foo", "bar")]).unwrap();
+        let inserted_trace_state = trace_state.insert("testkey", "testvalue").unwrap();
+        assert!(trace_state.get("testkey").is_none()); // The original state doesn't change
+        assert_eq!(inserted_trace_state.get("testkey").unwrap(), "testvalue"); //
+    }
 }
